(***********************************************************************)
(*                                                                     *)
(*                               Options                               *)
(*                                                                     *)
(*             projet Estime & Cristal, INRIA-Rocquencourt             *)
(*                                                                     *)
(*  Copyright 2005-2006 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: my_options.mli,v 1.1 2007/01/17 23:36:00 fclement Exp $ *)

val aref : 'a -> ('a -> unit) * (unit -> 'a);;
(** Abstract reference.
   [aref default] returns a couple of functions, the first one sets a variable
   with a default value [default], and the second one gets its value. *)

val saref : string -> 'a -> ('a -> unit) * (unit -> 'a);;
(** Single assignment abstract reference.
   Same as [aref] but the variable can be set only once. *)

val uaref : string -> 'a -> ('a -> unit) * (unit -> 'a);;
(** Uniquely defined abstract reference.
   [uaref vname default] returns a couple of functions, the first one sets, only
   once, a variable of name [vname], and the second one gets its value if and
   only if it has already been set. The default value [default] is used for
   initialization. *)

val optref : string -> ('a -> unit) * (unit -> 'a);;
(** Optional abstract reference.
   [optref vname] returns a couple of functions, the first one sets a variable
   of name [vname], and the second one gets its current value only if it has
   already been set. The abstract reference can be assigned more than once. *)

val malref : 'a list -> ('a -> unit) * (unit -> 'a list);;
(** Multiple argument list reference.
   [malref default_list] returns a couple of functions, the first one adds an
   element to an initially empty list, and the second one gets the list, which
   is the default list [default-list] if no element has been added. *)

val man_with_default : string -> string -> string;;

val man_with_default_and_valid : string -> string -> string -> string;;
