(*************************************************************************)
(*                                                                       *)
(*                                  Bh                                   *)
(*                                                                       *)
(*               Pierre Weis, projet Cristal, INRIA Rocquencourt         *)
(*                                                                       *)
(*  Copyright 2001, 2006 Institut National de Recherche en Informatique  *)
(*  et en Automatique.  All rights reserved.  This file is distributed   *)
(*  only by permission.                                                  *)
(*                                                                       *)
(*************************************************************************)

(* $Id: versionning.ml,v 1.3 2007/01/25 15:53:08 fclement Exp $ *)

(** Handling versionning generically. *)

type software_info = {
 software_name : string;
 software_banner_name : string;
 software_major_version : int;
 software_minor_version : int;
 software_patch_level : int;
 software_dev_level : int;
 software_dev_info : string;
 software_version_date : string;
 software_first_year : int;
};;

let make_patch_level si =
  let patch_level = si.software_patch_level in
  if patch_level = 0 then "" else Printf.sprintf ".%d" patch_level
;;

let make_short_version si =
  Printf.sprintf "%d.%d%s"
    si.software_major_version
    si.software_minor_version
    (make_patch_level si)
;;

let make_dev_info si =
  let dev_info = si.software_dev_info in
  if dev_info = "" then "" else Printf.sprintf ", %s" dev_info
;;

let make_long_version si =
  Printf.sprintf ("%s+dev%d (%s%s)")
    (make_short_version si)
    si.software_dev_level
    si.software_version_date
    (make_dev_info si)
;;

let make_banner si version =
  Printf.sprintf "The %s, version %s" si.software_banner_name version
;;

let make_print_version make_version si =
  let version = make_version si in
  let banner = make_banner si version in
  (fun () ->
    prerr_endline banner;
    exit 0);;

let make_print_short_version = make_print_version make_short_version;;
let make_print_long_version = make_print_version make_long_version;;

(* Adding command line options. *)
let short_option_name si =
  let sn = si.software_name in
  if String.length sn = 0 then '-' else sn.[0]
;;

let add_version_options si =
  Command_options.add
    "--version"
    (Arg.Unit (make_print_long_version si))
    (Printf.sprintf
       " %s detailed version information."
       si.software_banner_name);
  Command_options.add
    "-v"
    (Arg.Unit (make_print_short_version si))
    (Printf.sprintf
       " %s short version information."
       si.software_banner_name)
;;

let add_library_version_options si =
  Command_options.add
    (Printf.sprintf "--%s-version" si.software_name)
    (Arg.Unit (make_print_long_version si))
    (Printf.sprintf
       " %s detailed version information."
       si.software_banner_name);
  Command_options.add
    (Printf.sprintf "-%cv" (short_option_name si))
    (Arg.Unit (make_print_short_version si))
    (Printf.sprintf
       " %s short version information."
       si.software_banner_name)
;;
